--[[==========================================
	Recount
	==========================================
	A damage meter addon.
  ]]--


Recount = {}
Recount.name = "Recount"
Recount.command = "/recount"
Recount.versionString = "v0.3.2"
Recount.versionSettings = 9;
Recount.versionBuild = 24;

Recount.time = 0;
Recount.timeLastCombatEvent = 0;
Recount.timeLastCombat = 0;
Recount.onceHidden = false;
Recount.isDebugMode = false;
Recount.preMinMode = RC_MODE_COMBAT_LOG;
Recount.isResizing = false;
Recount.preCombatMode = RC_MODE_DAMAGE_DONE;

local EVENT_TYPE_IGNORE = 0;
local EVENT_TYPE_DAMAGE = 1;
local EVENT_TYPE_HEAL = 2;
local EVENT_TYPE_DAMAGE_DEFLECTED = 3;

local RC_MODE_DAMAGE_DONE = 0;
local RC_MODE_DAMAGE_DONE_TOTAL = 1;
local RC_MODE_DAMAGE_TAKEN = 2;
local RC_MODE_DAMAGE_TAKEN_TOTAL = 3;
local RC_MODE_HEALING_DONE = 4;
local RC_MODE_HEALING_DONE_TOTAL = 5;
local RC_MODE_HEALING_TAKEN = 6;
local RC_MODE_HEALING_TAKEN_TOTAL = 7;
local RC_MODE_COMBAT_LOG = 8;
local RC_MODE_EOL = 9;
local RC_MODE_MINIMIZED = 10;


local modeStrings = {
	[RC_MODE_DAMAGE_DONE] = "Damage Done last Fight",
	[RC_MODE_DAMAGE_DONE_TOTAL] = "Total Damage Done",
	[RC_MODE_DAMAGE_TAKEN] = "Damage Taken last Fight",
	[RC_MODE_DAMAGE_TAKEN_TOTAL] = "Total Damage Taken",
	[RC_MODE_HEALING_DONE] = "Healing Done last Fight",
	[RC_MODE_HEALING_DONE_TOTAL] = "Total Healing Done",
	[RC_MODE_HEALING_TAKEN] = "Healing Taken last Fight",
	[RC_MODE_HEALING_TAKEN_TOTAL] = "Total Healing Taken",	
	[RC_MODE_COMBAT_LOG] = "Combat Log",	
	[RC_MODE_MINIMIZED] = "n/a",	
}


local UI_STATUSBAR_MAX = 32;
local UI_STATUSBAR_HEIGHT = 25;
local UI_TITLEBAR_HEIGHT = 25;

Recount.settingsDefaults = {
	numberFormat = 1;
	currentMode = RC_MODE_DAMAGE_DONE_TOTAL;
	numStoredSessions = 10;
	autoSwitchMode = true;
	
	minimizeDuringCombat = false;
	
	hideOutOfCombat = false;
	hideOutOfCombatDelayInSeconds = 15;
	suspendDataCollectionWhileHidden = false;

	countDeflectedDmg = false;
	
	progressBarHeight = 25;
	progressBarFont = "ZoFontBoss";
	progressBarColor = {
		r = 0.8,
		g = 0.8,
		b = 0.5,
		a = 0.85
	};

	
	wndMain = {
		x = 10;
		y = 10;
		width = 250;
		height = 160;
		isVisible = true;
	}
}

-- contains elements similar to playerUnit, unit 1 is always the player itself
Recount.trackedUnits = {};

--[[==========================================
	playerUnit
	Prototype for the unit structure
	==========================================]]--
playerUnit = {
	displayName = string.gsub( GetUnitName( 'player' ) , "%-", "%%%-");

	sessionList = {};
	sessionCurrent = 1;
	sessionNum = 0;
	session = nil;

	total = {
		damageDone = 0;
		damageTaken = 0;
		healingDone = 0;
		healingTaken = 0;

		dps = 0;
		hps = 0;

		timeStart = 0;
		timeSpan = 0;

		damagePerAbility = {};
		healPerAbility = {};
	};
}


--[[==========================================
	Initialize
	==========================================]]--
function Recount.Initialize( eventCode, addOnName )

	-- Only set up for Recount
	if ( addOnName ~= Recount.name ) then
		return
	end

	Recount.settings = ZO_SavedVars:New( "RecountSettings" , Recount.versionSettings, nil, Recount.settingsDefaults, nil );
		
	EVENT_MANAGER:RegisterForEvent( Recount.name, EVENT_COMBAT_EVENT, Recount.CombatEvent );	
	EVENT_MANAGER:RegisterForEvent( Recount.name, EVENT_PLAYER_COMBAT_STATE, Recount.CombatStateEvent );	
	
	SLASH_COMMANDS[Recount.command] = Recount.SlashCommands;

	Recount.InitUI();

	-- never start in minimized mode
	if ( Recount.settings.currentMode == RC_MODE_MINIMIZED ) then
		Recount.settings.currentMode = RC_MODE_COMBAT_LOG;
	end
	Recount.SetMode( Recount.settings.currentMode );
		
	-- first tracked unit it always the player
	Recount.trackedUnits[1] = playerUnit;	

	-- set up the addon settings menu
	Recount.SettingsMenu.Initialize();
		
	Recount.CombatLogMsg( string.format( "Recount (%s) loaded.", Recount.versionString ) );
	Recount.CombatLogMsg( "Press the 'M' button in the top right corner to switch between modes." );
	Recount.CombatLogMsg( "Press the 'R' button in the top right corner to reset Recount." );
end

-- Hook initialization onto the ADD_ON_LOADED event
EVENT_MANAGER:RegisterForEvent( Recount.name, EVENT_ADD_ON_LOADED, Recount.Initialize )


--[[==========================================
	FormatNumber

	Shortens large integers into ..k or ..m text
	==========================================]]--
function Recount.FormatNumber( number )
	if number then
		if Recount.settings.numberFormat == 1 then
			if number > 999999 then
				return 	("%02.2fM"):format(number / 1000000);
			elseif number > 999 then
				return 	("%02.1fK"):format(number / 1000);
			else
				return "" .. math.floor(number);
			end
		else
			return "" .. math.floor(number);
		end
	else
		return ""
	end
end


function Recount.InitUI()

	-- RecountUI:SetResizeHandleSize( MOUSE_CURSOR_RESIZE_NS );

	if ( Recount.settings.wndMain.width < 10 ) then
		Recount.settings.wndMain.width = Recount.settingsDefaults.wndMain.width;
	end

	if ( Recount.settings.wndMain.x < 0 ) then
		Recount.settings.wndMain.x = 0;
	end

	if ( Recount.settings.wndMain.x > GuiRoot:GetWidth() ) then
		Recount.settings.wndMain.x = GuiRoot:GetWidth() - Recount.settings.wndMain.width;
	end

	RecountUI:SetAnchor( TOPLEFT, GuiRoot, TOPLEFT, Recount.settings.wndMain.x, Recount.settings.wndMain.y );	
	RecountUI:SetWidth( Recount.settings.wndMain.width );
	RecountUI:SetHeight( Recount.settings.wndMain.height );

	for i = 1, UI_STATUSBAR_MAX do
		local statusBar = CreateControlFromVirtual( "Statusbar", RecountUI, "StatusbarTemplate", i );
		_G["Statusbar"..i]:SetAnchor( TOPLEFT, RecountUI, TOPLEFT, 0, i * UI_STATUSBAR_HEIGHT );
	end
	
	RecountUIDebug:SetHidden( true );

	if ( not Recount.settings.wndMain.isVisible ) then
		Recount.Hide();
	end

	Recount.FitUIElements();

end

function Recount.MinimizeWindow()
	if ( Recount.settings.currentMode == RC_MODE_MINIMIZED ) then
		return;
	end
	Recount.preMinMode = Recount.settings.currentMode;
	Recount.SetMode( RC_MODE_MINIMIZED );
end

function Recount.RestoreWindow()
	Recount.SetMode( Recount.preMinMode );
end

function Recount.OnResizeStart( self )
	Recount.isResizing = true;
end

function Recount.OnMoveStop( self )
	Recount.settings.wndMain.x = self:GetLeft();
	Recount.settings.wndMain.y = self:GetTop();
	Recount.settings.wndMain.width = self:GetWidth();
	Recount.settings.wndMain.height = self:GetHeight();
end

function Recount.OnResizeStop( self )
	Recount.OnMoveStop( self );
	Recount.FitUIElements();
	Recount.isResizing = false;	
end

function Recount.GetMinWidth()
	-- three buttons + margin
	return 25 + 25 + 25 + 25;
end

function Recount.GetMinHeight()
	-- titlebar + statusbar + 1 progressbar
	return UI_STATUSBAR_HEIGHT + UI_TITLEBAR_HEIGHT + Recount.settings.progressBarHeight;
end

function Recount.FitStatusBars()

	-- calculate how many bars are visible
	local clientHeight = RecountUI:GetHeight() - UI_TITLEBAR_HEIGHT - UI_STATUSBAR_HEIGHT;
	local numBars = clientHeight / Recount.settings.progressBarHeight;

	for i = 1, UI_STATUSBAR_MAX do		
		_G["Statusbar"..i]:SetAnchor( TOPLEFT, RecountUI, TOPLEFT, 0, (i-1) * Recount.settings.progressBarHeight + UI_TITLEBAR_HEIGHT );
		_G["Statusbar"..i]:SetWidth( Recount.settings.wndMain.width );
		_G["Statusbar"..i]:SetHeight( Recount.settings.progressBarHeight );
		_G["Statusbar"..i]:SetColor( Recount.settings.progressBarColor.r, Recount.settings.progressBarColor.g, Recount.settings.progressBarColor.b, Recount.settings.progressBarColor.a );
		_G["Statusbar"..i.."_lblLeft"]:SetFont( Recount.settings.progressBarFont );
		if ( i > numBars ) then
			_G["Statusbar"..i]:SetHidden( true );
		else
			_G["Statusbar"..i]:SetHidden( false );
		end
	end

end

function Recount.FitUIElements()

	local w = RecountUI:GetWidth();
	local h = RecountUI:GetHeight();

	if ( w < Recount.GetMinWidth() ) then
		w = Recount.GetMinWidth();
		RecountUI:SetWidth( w );		
	end
	if ( h < Recount.GetMinHeight() ) then
		h = Recount.GetMinHeight();
		RecountUI:SetHeight( h );		
	end
	
	RecountUI_Title:SetWidth( w );
	RecountUI_lblStatus:SetWidth( w );

	RecountUI_lblCombatLogBG:SetWidth( w );
	RecountUI_lblCombatLogBG:SetHeight( h - UI_STATUSBAR_HEIGHT );

	RecountUI_lblCombatLog:SetWidth( w );
	RecountUI_lblCombatLog:SetHeight( h - UI_STATUSBAR_HEIGHT - UI_TITLEBAR_HEIGHT );

	Recount.FitStatusBars();

end

--[[==========================================
	UpdateUI
	==========================================]]--
function Recount.UpdateUI()
	
	if ( playerUnit.session ~= nil ) then
		RecountUI_lblStatus:SetText( string.format( "DPS: |ccccc55 %s|r   /   HPS: |c55cc55 %s|r  [%ds]", Recount.FormatNumber( playerUnit.session.dps ), Recount.FormatNumber( playerUnit.session.hps ), playerUnit.session.timeSpan/1000 ) );
	else
		RecountUI_lblStatus:SetText( string.format( "DPS: |ccccc55 %s|r   /   HPS: |c55cc55 %s|r", Recount.FormatNumber( playerUnit.total.dps ), Recount.FormatNumber( playerUnit.total.hps ) ) );
	end

	Recount.UpdateModeUI();

end

function Recount.OnUpdate( self )

	if ( Recount.isResizing ) then
		Recount.FitUIElements()		
	end

end

--[[==========================================

	==========================================]]--
function Recount.CombatLogMsg( msg )
	
	RecountUI_lblCombatLog:AddMessage( msg );
	--textCombatLog = string.sub( textCombatLog, 1, 300 );
	--textCombatLog = textCombatLog .."\n".. msg;
	
end


function spairs(t, order)
    -- collect the keys
    local keys = {}
    for k in pairs(t) do keys[#keys+1] = k end

    -- if order function given, sort by it by passing the table and keys a, b,
    -- otherwise just sort the keys 
    if order then
        table.sort(keys, function(a,b) return order(t, a, b) end)
    else
        table.sort(keys)
    end

    -- return the iterator function
    local i = 0
    return function()
        i = i + 1
        if keys[i] then
            return keys[i], t[keys[i]]
        end
    end
end

--[[==========================================

	==========================================]]--
function Recount.CombatStateEvent( eventCode, inCombat )
	
	if ( inCombat == true ) then
		Recount.CombatLogMsg( "+++ Combat +++" );

		--if ( Recount.settings.hideOutOfCombat ) then
			Recount.Show();
		--end
		Recount.onceHidden = false;
		if ( Recount.settings.autoSwitchMode ) then
			Recount.preCombatMode = Recount.settings.currentMode;
			Recount.SetMode( RC_MODE_COMBAT_LOG );
		end
		if ( Recount.settings.minimizeDuringCombat ) then
			Recount.MinimizeWindow();
		end

	else

		-- update Recount time pointer		
		Recount.time = GetGameTimeMilliseconds();
		Recount.timeLastCombat = Recount.time;

		if ( Recount.settings.minimizeDuringCombat ) then
			Recount.RestoreWindow();
		end
		if ( Recount.settings.autoSwitchMode ) then
			Recount.SetMode( Recount.preCombatMode );
		end		

		if ( playerUnit.session ~= nil ) then
			Recount.CombatLogMsg( string.format( "--- Combat [%ds] ---", playerUnit.session.timeSpan/1000 ) ); 
			Recount.CombatLogMsg( string.format( "--- out: |ccccc55 %s|r / |c55cc55 %s|r; in: |c883333 %s|r dmg",
								  Recount.FormatNumber( playerUnit.session.damageDone ), Recount.FormatNumber( playerUnit.session.healingDone ), Recount.FormatNumber( playerUnit.session.damageTaken ) ) );
		else
			Recount.CombatLogMsg( "--- Combat [%ds] ---" ); 
		end
	end

	Recount.UpdateUI();

end


--[[==========================================
	CheckIgnoreCombatEvent

	returns EVENT_TYPE_*
	==========================================]]--
function Recount.CheckIgnoreCombatEvent( result, isError, hitValue, powerType, damageType )

	-- ignore non-successful casts
	if ( isError ) then
		return EVENT_TYPE_IGNORE;
	end

	if ( hitValue == 0 ) then
		return EVENT_TYPE_IGNORE;
	end
	
	-- ignore certain powers
	if ( powerType == POWERTYPE_INVALID or powerType == POWERTYPE_MOUNT_STAMINA ) then
		return EVENT_TYPE_IGNORE;
	end
	
	if ( damageType == DAMAGE_TYPE_NONE ) then
		return EVENT_TYPE_IGNORE;
	end

	if ( result == ACTION_RESULT_HEAL or		
		 result == ACTION_RESULT_CRITICAL_HEAL or
		 result == ACTION_RESULT_HOT_TICK or
		 result == ACTION_RESULT_HOT_TICK_CRITICAL ) then
		return EVENT_TYPE_HEAL;
	end 

	if ( result == ACTION_RESULT_BLOCKED or
		 result == ACTION_RESULT_DAMAGE_SHIELDED or
		 result == ACTION_RESULT_PARRIED or
		 result == ACTION_RESULT_REFLECTED or
		 result == ACTION_RESULT_IMMUNE ) then
		if ( Recount.settings.countDeflectedDmg ) then
			return EVENT_TYPE_DAMAGE;
		end
		return EVENT_TYPE_DAMAGE_DEFLECTED;
	end 

	if ( result == ACTION_RESULT_ABSORBED or
		 result == ACTION_RESULT_BLOCKED_DAMAGE or
		 result == ACTION_RESULT_CRITICAL_DAMAGE or
		 result == ACTION_RESULT_DAMAGE or
		 result == ACTION_RESULT_DOT_TICK or
		 result == ACTION_RESULT_DOT_TICK_CRITICAL or
		 result == ACTION_RESULT_FALL_DAMAGE or
		 result == ACTION_RESULT_PARTIAL_RESIST or
		 result == ACTION_RESULT_PRECISE_DAMAGE or
		 result == ACTION_RESULT_WRECKING_DAMAGE ) then
		return EVENT_TYPE_DAMAGE;
	end 
	
	-- if ( powerType == POWERTYPE_STAMINA or powerType == POWERTYPE_FINESSE or powerType == POWERTYPE_MAGICKA ) then
		-- return true;
	-- end

	-- blacklist certain abilities ("Sprint Drain")
	-- if ( abilityName == "Sprint Drain" ) then
		-- return true;
	-- end
	
	return EVENT_TYPE_IGNORE;
end

--[[

DamageType
-----
DAMAGE_TYPE_COLD
DAMAGE_TYPE_DISEASE
DAMAGE_TYPE_DROWN
DAMAGE_TYPE_EARTH
DAMAGE_TYPE_FIRE
DAMAGE_TYPE_GENERIC
DAMAGE_TYPE_MAGIC
DAMAGE_TYPE_NONE
DAMAGE_TYPE_OBLIVION
DAMAGE_TYPE_PHYSICAL
DAMAGE_TYPE_POISON
DAMAGE_TYPE_SHOCK


CombatMechanicType
-----
POWERTYPE_FINESSE
POWERTYPE_HEALTH
POWERTYPE_INVALID
POWERTYPE_MAGICKA
POWERTYPE_MOUNT_STAMINA
POWERTYPE_STAMINA
POWERTYPE_ULTIMATE
POWERTYPE_WEREWOLF


CombatUnitType
-----
COMBAT_UNIT_TYPE_GROUP
COMBAT_UNIT_TYPE_NONE
COMBAT_UNIT_TYPE_OTHER
COMBAT_UNIT_TYPE_PLAYER
COMBAT_UNIT_TYPE_PLAYER_PET

]]--

function Recount.DebugStringifyUnitType( unitType )
	if ( unitType == COMBAT_UNIT_TYPE_PLAYER ) then
		return "CUT_PLAYER";
	elseif ( unitType == COMBAT_UNIT_TYPE_PLAYER_PET ) then
		return "CUT_PLAYER_PET";
	elseif ( unitType == COMBAT_UNIT_TYPE_GROUP ) then
		return "CUT_GROUP";
	elseif ( unitType == COMBAT_UNIT_TYPE_OTHER ) then
		return "CUT_OTHER";
	elseif ( unitType == COMBAT_UNIT_TYPE_NONE ) then
		return "CUT_NONE";
	else
		return "CUT_UNKNOWN";
	end
end


--[[==========================================
	CombatEvent

	Message handler
	==========================================]]--
function Recount.CombatEvent( eventCode, result, isError, abilityName, abilityGraphic, abilityActionSlotType, sourceName, sourceType, targetName, targetType, hitValue, powerType, damageType, log )

	-- check if we collect data in background
	if ( Recount.settings.suspendDataCollectionWhileHidden and Recount.settings.wndMain.isVisible == false ) then
		return;
	end

	-- update Recount time pointer
	Recount.time = GetGameTimeMilliseconds();
	
	-- check whether to hide the window
	if ( Recount.settings.wndMain.isVisible and Recount.settings.hideOutOfCombat and Recount.timeLastCombat + Recount.settings.hideOutOfCombatDelayInSeconds * 1000 < Recount.time ) then
		if ( Recount.onceHidden == false ) then
			Recount.onceHidden = true;
			Recount.Hide();
		end
	end	

	-- check if we should care about this combat event
	if ( sourceType ~= COMBAT_UNIT_TYPE_PLAYER and targetType ~= COMBAT_UNIT_TYPE_PLAYER and
		 sourceType ~= COMBAT_UNIT_TYPE_PLAYER_PET and targetType ~= COMBAT_UNIT_TYPE_PLAYER_PET and
		 sourceType ~= COMBAT_UNIT_TYPE_GROUP and targetType ~= COMBAT_UNIT_TYPE_GROUP ) then
		return;
	end

	if ( Recount.isDebugMode ) then
		debugString = sourceName .." [".. Recount.DebugStringifyUnitType( sourceType ) .."] <==> ".. targetName.. " [".. Recount.DebugStringifyUnitType( targetType ) .."] == (".. abilityName .."; ".. hitValue ..")<== ".. "; ".. abilityActionSlotType .."; ".. powerType .."; ".. damageType .. "; ".. result;
		RecountUIDebug_lblCombatLog:AddMessage( debugString );
	end
	
	eventType = Recount.CheckIgnoreCombatEvent( result, isError, hitValue, powerType, damageType );
	if ( eventType == EVENT_TYPE_IGNORE ) then
		return;
	end

	-- use the nice player name
	if ( sourceType == COMBAT_UNIT_TYPE_PLAYER ) then
		sourceName = playerUnit.displayName;
		Recount.timeLastCombat = Recount.time;
	end
	if ( targetType == COMBAT_UNIT_TYPE_PLAYER ) then
		targetName = playerUnit.displayName;
		Recount.timeLastCombat = Recount.time;
	end
			
	-- start a new session after 5 seconds without logged combat event		
	if ( Recount.time - Recount.timeLastCombatEvent > 5000) then
		-- Recount.Reset();
		Recount.StartNewSession()
	end

	Recount.timeLastCombatEvent = Recount.time;
	playerUnit.total.timeSpan = Recount.timeLastCombatEvent - playerUnit.total.timeStart;
	playerUnit.session.timeSpan = Recount.timeLastCombatEvent - playerUnit.session.timeStart;


	-- Player is the TARGET
	if ( targetType == COMBAT_UNIT_TYPE_PLAYER ) then
	
		if ( eventType == EVENT_TYPE_DAMAGE ) then				
			UnitDamageTaken( 1, abilityName, sourceName, hitValue, result );
		elseif ( eventType == EVENT_TYPE_HEAL ) then
			UnitHealingTaken( 1, abilityName, sourceName, hitValue, result );
		end
	end

	-- Player/Pet is the SOURCE
	if ( sourceType == COMBAT_UNIT_TYPE_PLAYER or sourceType == COMBAT_UNIT_TYPE_PLAYER_PET ) then
	
		if ( eventType == EVENT_TYPE_HEAL ) then
			UnitHealingDone( 1, abilityName, targetName, hitValue, result );		
		elseif ( eventType == EVENT_TYPE_DAMAGE ) then
			UnitDamageDone( 1, abilityName, targetName, hitValue, result );			
		end
	end
		
	Recount.UpdateUI();

end

function AddAbilityValue( abilityList, abilityName, hitValue )

	if ( abilityList[abilityName] == nil ) then
		abilityList[abilityName] = hitValue;
	else
		abilityList[abilityName] = abilityList[abilityName] + hitValue;
	end

end


--[[==========================================
	UnitDamageDone
	==========================================]]--
function UnitDamageDone( unitID, abilityName, targetName, hitValue, result )
	
	local u = Recount.trackedUnits[unitID];

	u.total.damageDone = u.total.damageDone + hitValue;
	u.session.damageDone = u.session.damageDone + hitValue;

	AddAbilityValue( u.total.damagePerAbility, abilityName, hitValue );
	AddAbilityValue( u.session.damagePerAbility, abilityName, hitValue );

	if ( u.total.timeSpan ~= 0 ) then
		u.total.dps = u.total.damageDone / ( u.total.timeSpan / 1000 );
	end

	if ( u.session.timeSpan ~= 0 ) then
		u.session.dps = u.session.damageDone / ( u.session.timeSpan / 1000 );
	end

	if ( result == ACTION_RESULT_CRITICAL_DAMAGE ) then
		eventString = string.format( "|ccccc55 *%d*|r |caaaa44 %s|r %s", hitValue, targetName, abilityName );
	else
		eventString = string.format( "|ccccc55 %d|r |caaaa44 %s|r %s", hitValue, targetName, abilityName );
	end

	Recount.CombatLogMsg( eventString );

end

--[[==========================================
	UnitDamageTaken
	==========================================]]--
function UnitDamageTaken( unitID, abilityName, sourceName, hitValue, result )
	
	local u = Recount.trackedUnits[unitID];

	u.total.damageTaken = u.total.damageTaken + hitValue;
	u.session.damageTaken = u.session.damageTaken + hitValue;

	--if ( result == ACTION_RESULT_CRITICAL_DAMAGE ) then
		--eventString = string.format( "|ccccc55 *%d*|r |caaaa44 %s|r %s", hitValue, targetName, abilityName );
	--else
		eventString = string.format( "|ccc4444 %d|r |c883333 %s|r %s", hitValue, sourceName, abilityName );
	--end

	Recount.CombatLogMsg( eventString );

end

--[[==========================================
	UnitHealingDone
	==========================================]]--
function UnitHealingDone( unitID, abilityName, targetName, hitValue, result )
	
	local u = Recount.trackedUnits[unitID];

	u.total.healingDone = u.total.healingDone + hitValue;
	u.session.healingDone = u.session.healingDone + hitValue;

	AddAbilityValue( u.total.healPerAbility, abilityName, hitValue );
	AddAbilityValue( u.session.healPerAbility, abilityName, hitValue );

	if ( u.total.timeSpan ~= 0 ) then
		u.total.hps = u.total.healingDone / ( u.total.timeSpan / 1000 );
	end

	if ( u.session.timeSpan ~= 0 ) then
		u.session.hps = u.session.healingDone / ( u.session.timeSpan / 1000 );
	end

	--if ( result == ACTION_RESULT_CRITICAL_DAMAGE ) then
	--	eventString = string.format( "|ccccc55 *%d*|r |caaaa44 %s|r %s", hitValue, targetName, abilityName );
	--else
		eventString = string.format( "|c44cc44 %d|r |c44aa44 %s|r %s", hitValue, targetName, abilityName );
	--end

	Recount.CombatLogMsg( eventString );

end

--[[==========================================
	UnitHealingTaken
	==========================================]]--
function UnitHealingTaken( unitID, abilityName, targetName, hitValue, result )
	
	local u = Recount.trackedUnits[unitID];

	u.total.healingTaken = u.total.healingTaken + hitValue;
	u.session.healingTaken = u.session.healingTaken + hitValue;

	-- AddAbilityValue( u.total.healTakenPerAbility, abilityName, hitValue );
	-- AddAbilityValue( u.session.healTakenPerAbility, abilityName, hitValue );

	--if ( result == ACTION_RESULT_CRITICAL_DAMAGE ) then
	--	eventString = string.format( "|ccccc55 *%d*|r |caaaa44 %s|r %s", hitValue, targetName, abilityName );
	--else
		eventString = string.format( "|c449944 %d|r |c447744 %s|r %s", hitValue, targetName, abilityName );
	--end

	Recount.CombatLogMsg( eventString );

end


function Recount.StartNewSession()

	-- currently only use one session
	-- playerUnit.sessionCurrent = playerUnit.sessionCurrent + 1;
	if ( playerUnit.sessionCurrent > Recount.settings.numStoredSessions ) then
		playerUnit.sessionCurrent = 1;		
	end
		
	playerUnit.sessionList[ playerUnit.sessionCurrent ] = {
		damageDone = 0;
		damageTaken = 0;
		healingDone = 0;
		healingTaken = 0;

		dps = 0;
		hps = 0;

		timeStart = Recount.time;
		timeSpan = 1;

		damagePerAbility = {};
		healPerAbility = {};
	}

	playerUnit.session = playerUnit.sessionList[ playerUnit.sessionCurrent ];

	Recount.ClearProgressBars();

end

function Recount.Reset()

	playerUnit.total.timeStart = Recount.time;	
	playerUnit.total.damageDone = 0;
	playerUnit.total.damageTaken = 0;
	playerUnit.total.healingTaken = 0;
	playerUnit.total.healingDone = 0;
	playerUnit.total.damagePerAbility = {};
	playerUnit.total.healPerAbility = {};

	playerUnit.sessionList = {};	
	playerUnit.sessionCurrent = 1;
	playerUnit.sessionNum = 0;
	Recount.StartNewSession();

	Recount.ClearProgressBars();

	Recount.CombatLogMsg( "==== RESET ====" );

end

function Recount.ClearProgressBars()

	for i = 1, UI_STATUSBAR_MAX do
		_G["Statusbar"..i]:SetValue( 0 );
		_G["Statusbar"..i.."_lblLeft"]:SetText( "" );
	end

end

function Recount.FillProgressBars( abilityTable, totalValue )

	if ( abilityTable == nil or totalValue == nil ) then
		return;
	end	
	
	local i = 1;
	for key,value in spairs( abilityTable, function(t,a,b) return t[b] < t[a] end ) do

		local abilityName = key;
		local dmgAbs = value;
		local dmgRel = dmgAbs / totalValue * 100;

		-- Recount.CombatLogMsg( ("------ %s: %d (%d %%)"):format( abilityName, dmgAbs, dmgRel ) );
									
		-- set progress bars with abilities in descending order
		if ( i <= UI_STATUSBAR_MAX ) then
			_G["Statusbar"..i]:SetValue( dmgRel );
			_G["Statusbar"..i.."_lblLeft"]:SetText( ("%s: |ceeeeee %s|r |c999999 (%d %%)|r"):format( abilityName, Recount.FormatNumber( dmgAbs ), dmgRel ) );
			_G["Statusbar"..i]:SetColor( 0.8 - i*0.05, 0.8 - i*0.05, 0.3, 0.8 );
		end

		i = i + 1;
	end

	Recount.FitStatusBars();

end

function Recount.FillSingleProgressBar( name, value )
	Statusbar1:SetValue( value );
	Statusbar1_lblLeft:SetText( ("%s: |ceeeeee %d|r"):format( name, value ) );
end

function Recount.UpdateModeUI()

	if ( Recount.settings.currentMode == RC_MODE_DAMAGE_DONE_TOTAL ) then
		Recount.FillProgressBars( playerUnit.total.damagePerAbility, playerUnit.total.damageDone );

	elseif ( Recount.settings.currentMode == RC_MODE_DAMAGE_DONE) then
		if ( playerUnit.session ~= nil ) then
			Recount.FillProgressBars( playerUnit.session.damagePerAbility, playerUnit.session.damageDone );
		end
		
	elseif ( Recount.settings.currentMode == RC_MODE_DAMAGE_TAKEN) then
		if ( playerUnit.session ~= nil ) then
			Recount.FillSingleProgressBar( "Damage Taken", playerUnit.session.damageTaken );
		end
		
	elseif ( Recount.settings.currentMode == RC_MODE_DAMAGE_TAKEN_TOTAL) then
		Recount.FillSingleProgressBar( "Damage Taken", playerUnit.total.damageTaken );

	elseif ( Recount.settings.currentMode == RC_MODE_HEALING_DONE) then
		if ( playerUnit.session ~= nil ) then
			Recount.FillProgressBars( playerUnit.session.healPerAbility, playerUnit.session.healingDone );
		end		

	elseif ( Recount.settings.currentMode == RC_MODE_HEALING_DONE_TOTAL) then
		Recount.FillProgressBars( playerUnit.total.healPerAbility, playerUnit.total.healingDone );
		
	elseif ( Recount.settings.currentMode == RC_MODE_HEALING_TAKEN) then
		if ( playerUnit.session ~= nil ) then
			Recount.FillSingleProgressBar( "Healing Taken", playerUnit.session.healingTaken );
		end
		
	elseif ( Recount.settings.currentMode == RC_MODE_HEALING_TAKEN_TOTAL) then
		Recount.FillSingleProgressBar( "Healing Taken", playerUnit.total.healingTaken );

	end

end

function Recount.SetMode( mode )

	if ( mode == RC_MODE_EOL ) then
		mode = RC_MODE_DAMAGE_DONE;
	end
	Recount.settings.currentMode = mode;
	Recount.ClearProgressBars();

	RecountUI_Title_Text:SetText( Recount.name .. " - " .. modeStrings[mode] );

	RecountUI_Title:SetHidden( false );
	RecountUI_lblCombatLogBG:SetHidden( false );

	if ( mode == RC_MODE_COMBAT_LOG ) then
		for i = 1, UI_STATUSBAR_MAX do
			_G["Statusbar"..i]:SetHidden( true );
		end
		RecountUI_lblCombatLog:SetHidden( false );	
		return;
	end	

	if ( mode == RC_MODE_MINIMIZED ) then
		for i = 1, UI_STATUSBAR_MAX do
			_G["Statusbar"..i]:SetHidden( true );
		end
		RecountUI_lblCombatLog:SetHidden( true );	
		RecountUI_Title:SetHidden( true );
		RecountUI_lblCombatLogBG:SetHidden( true );
		return;		
	end	

	for i = 1, UI_STATUSBAR_MAX do
		_G["Statusbar"..i]:SetHidden( false );
	end
	RecountUI_lblCombatLog:SetHidden( true );
	Recount.UpdateModeUI();

end

function Recount.TestProgressBars()
	Recount.SetMode( RC_MODE_DAMAGE_DONE );
	local test = {
		["Ability 1"] = 1000000,
		["Ability 2"] = 400000,
		["Ability 3"] = 50000,
		["Ability 4"] = 10000,
		["Ability 5"] = 1000,
		["Ability 6"] = 500,
		["Ability 7"] = 100,
		["Ability 8"] = 10,
	}
	Recount.FillProgressBars( test, 1460000 );
end

function Recount.ToggleNextMode()

	Recount.SetMode( Recount.settings.currentMode + 1 );

end

function Recount.OnClickedTitleButton( self, button )

	Recount.ToggleNextMode()

end

function Recount.OnClickedTitleButtonReset( self, button )
	Recount.Reset();
end

function Recount.OnClickedTitleButtonHide( self, button )
	Recount.Hide();
	d( "Type /recount show to show Recount again." );
end

function Recount.Hide()
	RecountUI:SetHidden( true );
	Recount.settings.wndMain.isVisible = false;
end

function Recount.Show()
	RecountUI:SetHidden( false );
	Recount.settings.wndMain.isVisible = true;
end


function Recount.SlashCommands( text )

	if ( text == "" ) then
		d( "Recount " .. Recount.versionString );
		d( "Command line options: hide; show; min; restore; reset;" );
		return
	end

	if ( text == "hide" ) then
		Recount.Hide();
		return
	end

	if ( text == "show" ) then
		Recount.Show();
		return
	end

	if ( text == "min" ) then
		Recount.MinimizeWindow()
		return
	end

	if ( text == "restore" ) then
		Recount.RestoreWindow()
		return
	end

	if ( text == "mode" ) then
		Recount.ToggleNextMode()
		return
	end

	if ( text == "reset" ) then
		Recount.Reset();
		return
	end

	if ( text == "debug" ) then
		if ( Recount.isDebugMode ) then
			RecountUIDebug:SetHidden( true );
			Recount.isDebugMode = false;
		else
			RecountUIDebug:SetHidden( false );
			Recount.isDebugMode = true;
		end
		return
	end
		
	if ( text == "test" ) then
		Recount.TestProgressBars();
		-- d( " " .. playerName );
		return
	end
end
